unit Modcode;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, LzExpand, Ver, Inifiles;

  function CopyFile(SrcFilenm, DestPath, DestFilenm: string): boolean;
  function VCopyFile(flags: word; Filename, SrcFileName, WinDir, DestPath: string):string;
  procedure CopyAbort;
  procedure CreateProgManGrp(GroupName, GroupPath: string);
  procedure CreateProgManItem(CmdLine, IconTitle, IconPath: string);
  procedure Main1;
  procedure Main2;
  procedure Main3;

var { Global varibles }
  WinVer, SourcePath, DestDrive, SourceDrive : String;
  iError, iInstalled: Integer;
  bError: Boolean;
  lError: LongInt;

const { Global constants }
  Win31size = 476578;
  Win95size = 479122;
  AppName = 'QwikEdit';
  ByLine = 'Version 1.02';
  DefaultDir = 'C:\QwikEdit';
  IniFileName = 'ReSetup.ini';
  IniPrefix = 'QwikInst.';

implementation
{$I-}

uses
  frmmaine, frmcopie, frmcngrt, frmwelcm, frmgtpth;

function CopyFile(SrcFilenm, DestPath, DestFilenm: string): boolean;
var
  tofDestFile, tofSrcFile: TOFStruct;
  hdlDestFile, hdlSrcFile: Integer;
  sIniKey: string;
  DestFilePath, SrcFilePath: array [0..255] of char;
  DFilePath, SFilePath: pchar;
  iSize: LongInt;
  f: file of byte;
begin
  if not FileExists(SourcePath + SrcFilenm) then
  begin
    MessageDlg('Oh dear, the source file has gone missing!',
      mtError, [mbAbort], 10004);
    CopyFile:=false;
    exit;
  end
  else
  begin
    { update status bar }
    frmCopy.lblDest.caption := DestPath + DestFilenm;
    frmCopy.lblSource.caption := SourcePath + SrcFilenm;
    frmCopy.Refresh;

    { what are the full filenames with path ?? AS C STYLE STRINGS }
    StrPCopy(DestFilePath, (DestPath + DestFilenm));
    StrPCopy(SrcFilePath, (SourcePath + SrcFilenm));
    { point to them }
    DFilePath := addr(DestFilePath);
    SFilePath := addr(SrcFilePath);

    { get file handles }
    hdlDestFile := OpenFile(DFilePath, tofDestFile, OF_CREATE);
    hdlSrcFile := OpenFile(SFilePath, tofSrcFile, OF_READWRITE or OF_CANCEL or OF_PROMPT);

    { copy the file }
    iError := CopyLZFile(hdlSrcFile, hdlDestFile);

    { let the computer do stuff }
    Application.ProcessMessages;

    { check for errors! }
    case iError of
      LZERROR_BADINHANDLE:
      begin
        MessageDlg('Bad Source File Handle', mtError, [mbOK],10020);
        CopyFile := false;
      end;
      LZERROR_BADOUTHANDLE:
      begin
        MessageDlg('Bad Destination File Handle', mtError, [mbOK],10020);
        CopyFile := false;
      end;
      LZERROR_BADVALUE:
      begin
        MessageDlg('Input value is out of range', mtError, [mbOK],10021);
        CopyFile := false;
      end;
      LZERROR_GLOBALLOC:
      begin
        MessageDlg('Insufficient memory', mtError, [mbOK], 10010);
        CopyFile := false;
      end;
      LZERROR_GLOBLOCK:
      begin
        MessageDlg('Invalid internal handle', mtError, [mbOK],10020);
        CopyFile := false;
      end;
      LZERROR_READ:
      begin
        MessageDlg('The source file format is incompatable', mtError, [mbOK], 10022);
        CopyFile := false;
      end;
      LZERROR_UNKNOWNALG:
      begin
        MessageDlg('Unrecognised compression algorithm', mtError, [mbOK], 10022);
        CopyFile := false;
      end;
      LZERROR_WRITE:
      begin
        MessageDlg('Insufficient disk space', mtError, [mbOK], 10011);
        CopyFile := false;
      end;
    else
      begin
        CopyFile := true;

        { Tell the install log about it }
        frmMain.IniFile.WriteString(IniPrefix+'File List', 'File' + IntToStr(iInstalled)
          , StrPas(DestFilePath));
        Inc(iInstalled);

        { get file size }
        AssignFile(f, DestFilePath);
        Reset(f);
        iSize := FileSize(f);
        CloseFile(f);

        { and add it to progress }
        frmCopy.gagProgress.Progress := frmCopy.gagProgress.Progress + iSize;
        frmCopy.gagProgress.Refresh;
        exit;
      end;
    end; { case statement }
  end; { huge if to determine whether source exists }
end; { function }

function VCopyFile(flags: word; Filename, SrcFileName, WinDir, DestPath: string):string;
var
  sIniKey, sIniValue: string;
  cFilename, cWinDir, cDestDir, cCurrDir, cSuggestedDir, cSrcFilename, cTmpFile: array [0..255] of char;
  cSourcePath: array [0..255] of char;
  TmpFileLen, SuggestLen: word;
  DFilePath, SFilePath: pchar;
  ForceInstall, TryAgain: boolean;
  Attributes: integer;
  iSize: LongInt;
  f: file of byte;
label
  lTryAgain;
begin
  { sort some varibles out }
  StrPCopy(cFilename, Filename);
  StrPCopy(cWinDir, WinDir);
  StrPCopy(cDestDir, DestPath);
  StrPCopy(cSrcFileName, SrcFileName);
  StrPCopy(cSourcePath, SourcePath);
  ForceInstall := False;
  TryAgain := False;
  SuggestLen := 260;
  TmpFileLen := 260;

  { find any original versions }
  VerFindFile(flags, cSrcFilename, cWinDir, cDestDir, cCurrDir, TmpFileLen,
    cSuggestedDir, SuggestLen);

  { update status bar }
  frmCopy.lblDest.caption := (StrPas(cSuggestedDir) + '\' + Filename);
  frmCopy.lblSource.caption := SourcePath + StrPas(cSrcFilename);
  frmCopy.Refresh;

lTryAgain:
  { physically copy the file }
  lError := VerInstallFile(0, cSrcFilename, cFilename, cSourcePath,
    cSuggestedDir, cCurrDir, cTmpFile, TmpFileLen);

  { let the computer do stuff }
  Application.ProcessMessages;

  { error checking }
  if (lError and VIF_SRCOLD) = VIF_SRCOLD then
  { trying to install older file over newer one }
  begin
    iError := MessageDlg('An older previous version already' +
      chr(10) + chr(13) + 'exists on your system, Overwrite',
      mtInformation, [mbOK, mbNo], 10030);
    if iError = mrOK then
      ForceInstall := true;
  end;

  if (lError and VIF_MISMATCH) = VIF_MISMATCH then
  { differing attributes }
  begin
    iError := MessageDlg('A mis-matching version already' +
      chr(10) + chr(13) + 'exists on your system, Overwrite',
      mtInformation, [mbOK, mbNo], 10031);
    if iError = mrOK then
      ForceInstall := true;
  end;

  if (lError and VIF_DIFFLANG) = VIF_DIFFLANG then
  { differing language }
  begin
    iError := MessageDlg('A version with a different language' +
      chr(10) + chr(13) + 'exists on your system, Overwrite',
      mtInformation, [mbOK, mbNo], 10032);
    if iError = mrOK then
      ForceInstall := true;
  end;

  if (lError and VIF_CANNOTREADSRC) = VIF_CANNOTREADSRC then
  { cannot read source file }
  begin
    iError := MessageDlg('Unable to read source, Retry?', mtError,
      [mbRetry, mbAbort], 10004);
    if iError = mrRetry then
    begin
      TryAgain := true;
    end
    else
    begin
      VCopyFile := 'Error';
      exit;
    end;
  end;

  if (lError and VIF_DIFFTYPE) = VIF_DIFFTYPE then
  { differing types }
  begin
    iError := MessageDlg('A version of different type already' +
      chr(10) + chr(13) + 'exists on your system, Overwrite',
      mtInformation, [mbOK, mbNo], 10033);
    if iError = mrOK then
      ForceInstall := true;
  end;

  if (lError and VIF_WRITEPROT) = VIF_WRITEPROT then
  { write protected destination }
  begin
    { un-set write protect bit and do it again }
    Attributes := FileGetAttr (StrPas(cSuggestedDir) + '\' + Filename);
    Attributes := Attributes and not faReadOnly;
    iError := FileSetAttr((StrPas(cSuggestedDir) + '\' + Filename), Attributes);
    if not (iError = 0) then
    begin
      { critical error, unable to un-write-protect dest file }
      MessageDlg('The destination file is write protected'
        + chr(13) + chr(10) + 'and I can''t change it', mtError, [mbAbort], 10012);
      VCopyFile := 'Error';
      exit;
    end;
  end;
  { end of error checking }

  if ForceInstall then
  begin
    TryAgain := false;
    lError := VerInstallFile(VIFF_FORCEINSTALL, cSrcFilename,
      cFilename, cSourcePath, cSuggestedDir, cCurrDir, cTmpFile,
      TmpFileLen);

    { error checking }
    if (lError and VIF_WRITEPROT) = VIF_WRITEPROT then
    { write protected destination }
    begin
      iError := MessageDlg('A pre-existing version is' +
        chr(10) + chr(13) + 'Write protected', mtError, [mbOK], 10012);
    end;
    { end of error checking }
  end;

  if TryAgain then goto lTryAgain;

  { Tell the install log about it }
  frmMain.IniFile.WriteString(IniPrefix+'File List', 'File' + IntToStr(iInstalled),
    StrPas(cSuggestedDir) + '\' + Filename);
  Inc(iInstalled);

  VCopyFile := StrPas(cSuggestedDir);

  { get file size }
  AssignFile(f, StrPas(cSuggestedDir) + '\' + Filename);
  Reset(f);
  iSize := FileSize(f);
  CloseFile(f);

  { and add it to progress }
  frmCopy.gagProgress.Progress := frmCopy.gagProgress.Progress + iSize;
  frmCopy.gagProgress.Refresh;

end; { function }

procedure CopyAbort;
var
  wQuestion: word;
begin
  wQuestion := MessageDlg('File not copied, Quit?'
    , mtConfirmation, [mbYes, mbNo], 1024);
  if wQuestion = mrYes then
  begin
    LZDone;
    frmMain.IniFile.Free;
    halt; { get out quick }
  end;
end;

procedure CreateProgManGrp(GroupName, GroupPath: string);
var
  Cmd: array [0..255] of char;
  Macro: string;
begin
  Macro := Format('[CreateGroup(%s)]', [GroupName]) + #13#10;
  StrPCopy (Cmd, Macro);
  if not frmMain.DDEClient.ExecuteMacro(Cmd, False) then
    MessageDlg('Unable to create group.', mtInformation, [mbOK], 10040);

  { Tell the install log about it }
  frmMain.IniFile.WriteString(IniPrefix+'System', 'CreateGroup', GroupName);

end;

procedure CreateProgManItem(CmdLine, IconTitle, IconPath: string);
var
  Cmd: array [0..255] of char;
  sIniKey, Macro: string;
begin
  frmCopy.lblDest.caption := IconTitle;
  frmCopy.Refresh;
  Macro := '[ReplaceItem('+IconTitle+')]'+chr(13)+chr(10);
  StrPCopy (Cmd, Macro);
  if not frmMain.DDEClient.ExecuteMacro(Cmd, False) then
    MessageDlg('Unable to replace program item: ' + IconTitle, mtInformation, [mbOK], 10041);

  Macro := '[AddItem('+CmdLine+','+IconTitle+','+IconPath+',0,0,0)]'+ chr(13)+chr(10);
  StrPCopy (Cmd, Macro);
  if not frmMain.DDEClient.ExecuteMacro(Cmd, False) then
    MessageDlg('Unable to add program item: ' + IconTitle, mtInformation, [mbOK], 10042);

  { Tell the install log about it }
  Inc(iInstalled);
  frmMain.IniFile.WriteString(IniPrefix+'Program Items',  'Item' + IntToStr(iInstalled),
    IconTitle);

  Macro := '[ShowGroup(Digital Curriculum Vitae, 1)]'+ chr(13)+chr(10);
  StrPCopy (Cmd, Macro);
  frmMain.DDEClient.ExecuteMacro(Cmd, False)
end;

procedure Main1;
var
  WinVersion: LongInt;

begin
  if frmMain.ClientWidth<552 then frmMain.ClientWidth := 552;

  { maximise form }
  frmMain.WindowState := wsMaximized;

  { move things around, set the labels }
  with frmMain do
  begin
    lblAppName.caption := AppName;
    lblAppNames.caption := AppName;
    Caption := AppName + ' Setup';
    lblByLine.caption := ByLine;
    btnExit.top:= (frmMain.ClientHeight-33);
    btnExit.left:= (frmMain.ClientWidth-97);
  end;

  {Get windows version etc.}
  WinVersion := GetVersion and $FFFF;
  WinVer:=Concat((IntToStr(Winversion mod 256)),'.',(IntToStr(Winversion div 256)));
  SourcePath := ExtractFilePath(Paramstr(0));
  SourceDrive := copy(Paramstr(0),1,3);

  { Create installation log }
  frmMain.IniFile := TInifile.Create(IniFileName);
  { Show first 'dialog' }
  frmWelcome.Show;
end;

procedure Main2;
begin
  frmWelcome.Free;
  { Tell the install log about this application }
  frmMain.IniFile.WriteString(IniPrefix+'System', 'AppName', AppName);
  frmMain.IniFile.WriteString(IniPrefix+'System', 'ByLine', ByLine);

  bError:=false;
  frmGetPath.Show;
end;

procedure Main3;
var
  WinVersion: LongInt;
  which: word;
  sProgItem, sIconPath, sProgPath, DestPath, WinPath: string;
  sWhereApp, sWhereUnSetup: string;
  cWinPath: array [0..255] of char;

begin
  frmGetPath.Hide;
  { check correct path }
  DestPath := frmGetPath.edtGetPath.text;
  { see if directory exists }
  ChDir(DestPath);
  if IOResult <> 0 then
  begin
    { create it if it doesn't }
    MkDir(DestPath);
    if IOResult <> 0 then
    begin
      MessageDlg('Cannot create directory', mtWarning, [mbOk], 10001);
      bError:= true;
    end
    else
      MessageDlg('New directory created', mtInformation, [mbOk], 10002);

      { Tell the install log about it }
      frmMain.IniFile.WriteString(IniPrefix+'System', 'CreateDirectory', DestPath);
    end
  else
  { The directory already exists so check that's okay }
  begin
    which := MessageDlg
      ('Directory alredy exists, do you want to use the same directory?',
      mtConfirmation, [mbYes, mbNo], 10003);
    if which = mrNo then
      bError:= true
  end;

{ Get windows directory }
GetWindowsDirectory(cWinPath, 260);
WinPath := StrPas(cWinPath);

{ for some reason I'm going to check for a trailing \ }
if (copy(WinPath, Length(WinPath), 1)) <> '\' then
AppendStr(WinPath, '\');

{ in the event of error prompt for path again }
if bError then
  begin
    { ask for path again }
    frmGetPath.ShowModal;
  end
else
  begin

    { for some reason I'm going to check for a trailing \ }
    if (copy(DestPath, Length(Destpath), 1)) <> '\' then
    AppendStr(Destpath, '\');

    { show copy progress 'dialog' }
    frmCopy.Show;
    frmCopy.lblDest.caption := DestPath;
    frmCopy.lblSource.caption := SourcePath;

    iError := LZStart;
    if iError <> 0 then
    begin  { copy the files }
      iInstalled := 0;

      { global files }
      sWhereUnSetup := VCopyFile(VFFF_ISSHAREDFILE,'unsetup.exe','unsetup.ex_',WinPath,
        DestPath); { uninstaller }
      if sWhereUnSetup = 'Error' then CopyAbort;
      sWhereApp := VCopyFile(0,'QwikEdit.exe','QwikEdit.ex_',WinPath,DestPath); { executable }
      if sWhereApp = 'Error' then CopyAbort;
      bError := CopyFile('readme.tx_',DestPath,'ReadMe.txt'); { readme }
      if not bError then CopyAbort;

      if WinVer = '3.95' then
      begin
        { windows 95 only files }
        bError := CopyFile('qwikedit.re_',DestPath,'QwikEdit.reg'); { regestration entries }
        if not bError then CopyAbort;
      end;

      { Tell the install log about it }
      frmMain.IniFile.WriteInteger(IniPrefix+'System', 'NumFiles', iInstalled);

      { tidy up }
      LZDone;
    end
    else
    begin
      MessageDlg('Insufficient Memory ?, Quitting...', mtError, [mbAbort], 10010);
      frmMain.IniFile.Free;
      Application.Terminate;
    end; { of if associated with LZStart error checking }

    { time to install program items }
    { update status }
    with frmCopy do
    begin
      lblDest.caption := '';
      lblSource.caption := 'Updating Program Manager Items';
      lblFrom.caption := '';
      lblTo.caption := '';
      Refresh;
    end;
    iInstalled := 0;

    { Do DDE operations }
    frmMain.DDEClient.Openlink;

     { Create Program Manager group }
     CreateProgManGrp(AppName+' '+ByLine, 'QWIKEDIT.GRP');

     { Create Program Items }
     { Uninstaller executable }
     sProgPath := sWhereUnSetup + '\UnSetup.EXE qwikinst.ini';
     sProgItem := 'Un-Installer for '+AppName;
     sIconPath := sWhereUnSetup + '\UnSetup.EXE';
     CreateProgManItem(sProgPath, sProgItem, sIconPath);

     { Main executable }
     sProgPath := sWhereApp + '\QwikEdit.EXE';
     sProgItem := 'QwikEdit 1.00';
     sIconPath := sWhereApp + '\QwikEdit.EXE';
     CreateProgManItem(sProgPath, sProgItem, sIconPath);

     { Read-me }
     sProgPath := DestPath + 'ReadMe.txt';
     sProgItem := 'Readme for QwikEdit 1.00';
     sIconPath := DestPath + 'ReadMe.txt';
     CreateProgManItem(sProgPath, sProgItem, sIconPath);

     if WinVer = '3.95' then
     begin
       { Windows 95 only files }
       { Regestration Entries }
       sProgPath := DestPath + 'QwikEdit.reg';
       sProgItem := 'Registry entries for Un-Installer and QwikEdit';
       sIconPath := DestPath + 'QwikEdit.reg';
       CreateProgManItem(sProgPath, sProgItem, sIconPath);
     end;

     { Finished DDE Operations }
    frmMain.DDEClient.Closelink;

    { Tell the install log about it }
    frmMain.IniFile.WriteInteger(IniPrefix+'System', 'NumProgItems', iInstalled);

    { Clear progress dialog as I'm done }
    frmCopy.close;
    frmCongrats.showModal;
    frmMain.Close;
  end;

end;

end.
